\\ Copyright 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp <dragon-curve.gp
\\
\\ A location in the Heighway/Harter dragon curve fractal is a change of
\\ base from binary to 1+I, with negate below each 01 bit pair.

default(strictargs,1);
default(recover,0);
read("../vecper.gp");

\\ f_to_z(f) takes a rational f in the range 0 <= f <= 1 and returns its
\\ location in the dragon curve fractal as a complex rational x + I*y.
\\
\\ In vecper_apply_state(), "state" is a vector [sign, prev_bit].
\\ sign = 1 or -1 to multiply the current bit.
\\ If prev_bit,bit = 0,1 then sign is flipped for subsequent bits.
\\ The result is a vecper with 0,1,-1 values.
\\ Those values are then "digits" of base b=1+I, at successive positions
\\ below the radix point.  vecper_frac_fromdigits() accepts digits which are
\\ +/-1 like this, and accepts a complex number base b too.  (It just powers
\\ up anything given.)
\\
\\ The inline "(bit,state)->" function here does not depend on any of the
\\ local variables so could be separated out as a global if desired.  That
\\ would be good if it had other uses, perhaps crunching a few bits of
\\ something irrational like a quad.  The inline form is a little harder to
\\ read, but doesn't add another global name.
\\
\\ vecper_frac_digits() which is the starting bits here accepts floating
\\ point f too.  The base 2 used here is the same as the binary of t_FLOAT
\\ so that v is its bits in the manner of binary(f).
\\
dragon_f_to_z(f) =
{
  my(v=vecper_frac_digits(f,2));
  v=vecper_apply_state
    ((bit,state)->
     my(s=state[1],                    \\ sign
        prev_bit=state[2],             \\ previous bit
        new_bit = bit*s);              \\ possible negate
     if(prev_bit==0 && bit==1, s=-s);  \\ negate below each 01
     [new_bit, [s,bit]]                \\ new bit and new state
     ,
     v,
     [1,0]);      \\ initial state [sign=1, prev_bit=0]
  vecper_frac_fromdigits(v, 1+I);
}

dragon_f_to_z(0) == 0                || error();
dragon_f_to_z(1/3) == 1/5 - 2/5*I    || error();
dragon_f_to_z(1/2) == 1/2 - 1/2*I    || error();
dragon_f_to_z(1/4) == -1/2*I         || error();
dragon_f_to_z(1) == 1                || error();

\\-----------------------------------------------------------------------------

\\ Plot the fractal using dragon_f_to_z() for coordinates.
\\ len is resolution steps 1/len to plot.
\\ If len is a power-of-2 then the familiar dragon grid is seen.
\\ If len not a power-of-2 then the fractional locations are a wilder
\\ selection of points within sub-curves.  The overall shape is right, but
\\ the inside steps are a bit strange.
{
  my(len=800,
     f_vec=vector(len+1,i, (i-1)/len),   \\ 0, 1/len, ..., len/len (=1)
     z_vec=apply(dragon_f_to_z, f_vec));
  plothraw(real(z_vec), imag(z_vec), 1);
}


\\-----------------------------------------------------------------------------
