\\ Copyright 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("../vecper.gp");


\\ Usage: gp <frac-digits.gp
\\
\\ This is some example calculations of fractions 0 <= f <= 1 converted to
\\ decimal digits, or digits of a given base.

\\-----------------------------------------------------------------------------

\\ 1/3 in decimal is .333...
vecper_frac_digits(1/3) == [ [], [3] ]            || error();

\\ 7/48 in decimal is .1458 333... so an initial fixed part
vecper_frac_digits(7/48) == [ [1,4,5,8], [3] ]    || error();

\\ 3/100 is .03 000...
vecper_frac_digits(3/100) == [ [0,3], [0] ]       || error();
\\
\\ Or ask for the 9s form .02 999...
vecper_frac_digits(3/100,,1) == [ [0,2], [9] ]    || error();
\\
\\ Or finite form .03 only.
vecper_frac_digits(3/100,,2) == [ [0,3], [] ]     || error();
\\
\\ The infinite form, with 0s, is the default since vecper is aimed at
\\ infinite periodic vectors and for digit twiddling or testing it's usually
\\ convenient to have 0s ready to examine.

\\ Integer 1 is always .999... since that is the only way to represent it
\\ using digits solely below the decimal point.
vecper_frac_digits(1) == [ [], [9] ]              || error();


\\-----------------------------------------------------------------------------

\\ 1/3 in binary is .010101...
vecper_frac_digits(1/3,2) == [ [], [0,1] ]          || error();

\\ 3/16 in binary is .0011 000...
vecper_frac_digits(3/16,2) == [ [0,0,1,1], [0] ]    || error();
\\
\\ Or the "9s" form is .0010 111...
\\ Here "9" means "base-1" which for binary is 2-1 = 1 bits
vecper_frac_digits(3/16,2,1) == [ [0,0,1,0], [1] ]  || error();

\\ 1 in binary is .111...
vecper_frac_digits(1,2) == [ [], [1] ]              || error();


\\-----------------------------------------------------------------------------

\\ .333... gives back 1/3
vecper_frac_fromdigits([[],[3]]) == 1/3          || error();
\\
\\ or .010101... binary likewise gives back 1/3
vecper_frac_fromdigits([[],[0,1]],2) == 1/3        || error();
\\
\\ The periodic part doesn't have to be in minimal form.
\\ Some extra initials or repetitions gives the same.
\\   . 0 1010         = 1/3
\\       \--/ repeat
vecper_frac_fromdigits([[0],[1,0,1,0]],2) == 1/3   || error();

\\ The 0s, 9s, or finite, forms of an exact integer/base^k fraction all come
\\ back the same.
vecper_frac_fromdigits([[3,7],[0]]) == 37/100      || error();  \\ 0s
vecper_frac_fromdigits([[3,6],[9]]) == 37/100      || error();  \\ 9s
vecper_frac_fromdigits([[3,7],[]])  == 37/100      || error();  \\ finite

\\ vecper_frac_fromdigits() allows any "digit" values, so for example
\\ negative at each digit is just negative the whole result.
\\
vecper_frac_fromdigits([[],[-3]]) == -1/3          || error();
\\
\\ Or binary 6/2 + 6/2^2 + 6/2^3 + ... = 6*(1/2 + 1/4 + ...) = 6
vecper_frac_fromdigits([[],[6]],2) == 6          || error();

\\ vecper_frac_fromdigits() allows any "base" too, so for example a
\\ fractional base 7/3 with all digits 2 is
\\     2/(7/3) + 2/(7/3)^2 + 2/(7/3)^3 + ...
\\     = 2/(7/3) * 1/(1 - 1/(7/3))
\\     = 3/2
vecper_frac_fromdigits([[],[2]],7/3) == 3/2          || error();

\\ The base can be a polynomial like "x" too, which gives a t_RFRAC and
\\ reveals the general calculation made for any base x.
\\     2/x + 2/x^2 + 2/x^3 + ...
\\     = 2/x * 1/(1 - 1/x)
\\     = 2/(x-1)
vecper_frac_fromdigits([[],[2]],x) == 2/(x-1)        || error();
\\
\\ This is like vecper_polrev(), except that vecper_polrev() begins with the
\\ first term of the vector as the constant term, not the "1" term.
vecper_frac_fromdigits([[],[2]],1/x) \
  == x * vecper_polrev([[],[2]],x)                   || error();

\\ vecper_frac_fromdigits() is a bit more general than vecper_polrev()
\\ actually, since vecper_frac_fromdigits() allows base to be anything at
\\ all which can be powered, summed, and inverted, whereas vecper_polrev()
\\ only takes the name of a variable (which you might later subst() to put
\\ more).
\\
vecper_frac_fromdigits([[],[2]], x^2) == 2/(x^2-1)               || error();
\\
\\ matrix as "base"
vecper_frac_fromdigits([[],[2]], [1,3; 1,0]) == [2/3,2; 2/3,0]   || error();

\\ For longer fixed and periodic parts, the general form is like
\\
vecper_frac_fromdigits([[a,b,c],[d,e,f]],x) == \
  a/x + b/x^2 + c/x^3  + 1/x^4 *(d + e/x + f/x^2)/(1 - 1/x^3)    || error();


\\-----------------------------------------------------------------------------
